@@ -2,7 +2,7 @@ from os import path
 import sys
 from werkzeug import Request, ClosingIterator, run_simple, SharedDataMiddleware
 from werkzeug.routing import Rule
-from werkzeug.exceptions import HTTPException
+from werkzeug.exceptions import HTTPException, NotFound
 from jinja2 import Environment, FileSystemLoader
 from buro.shared import local, local_manager
 from buro.urls import url_map
@@ -61,8 +61,8 @@ class Configuration(dict):
         for key, value in self.iteritems():
             txt += '\t%s => %s\n' % (key, value)
         return txt
-        
-        
+
+
         
 class Buro(object):
     """The Buro object implements the whole WSGI application: this is the main
@@ -124,13 +124,18 @@ class Buro(object):
             'function': self.function,
             'markup': self.markup,
         }
+
+        # Special Pages handlers
+        self.special_handlers = {
+            #'404': ('handler', kwargs),
+        }
         
         print self.config
-        
+
         
     def __call__(self, environ, start_response):
         return self._dispatch(environ, start_response)
-        
+    
     def _dispatch(self, environ, start_response):
         local.application = self
         request = Request(environ)
@@ -138,11 +143,21 @@ class Buro(object):
         try:
             endpoint, values = adapter.match()
             response = self._endpoint_manager(endpoint, request, values)
+        except NotFound, e:
+            stored_data = self.special_handlers.get('404', None)
+            if stored_data == None:
+                response = e
+            else:
+                handler = self.endpoints_handlers.get(stored_data[0], False)
+                if not handler:
+                    raise BuroError("No handler named %s" % data[0])
+                response = handler(stored_data[1], request)
+            response.status_code = 404
         except HTTPException, e:
             response = e
         return ClosingIterator(response(environ, start_response),\
                                         [local_manager.cleanup])
-        
+    
     def _endpoint_manager(self, endpoint, request, values):
         data = self.local_mapping.get(endpoint)
         
@@ -154,16 +169,92 @@ class Buro(object):
         info = data[1]
 
         return handler(info, request, **values)
+    
+    def set_special_handler(self, **kwargs):
+        """Set a handler for a special page. Special pages are returned when a
+        non 200 HTTP status code is returned (so 404, 500...). For now, only
+        404 is implemented.
         
+        Example::
+        
+            def view(request):
+                return Response('can haz cheezburger?')
+            
+            app.set_special_handler(name='404', handler='function', function=view)
+        
+        Now all your not found pages will print "can haz cheezburger?" instead
+        of the 404 page of your web browser.
+        """
+        name = kwargs.get('name', False)
+        if not name:
+            raise BuroError('You must provide the name of the special handler.')
+        
+        handler = kwargs.get('handler', False)
+        if not handler:
+            raise BuroError("Your special handler doesn't have a handler")
+        
+        if not handler in self.endpoints_handlers:
+            raise BuroError("%s is not a loaded handler." % handler)
+            
+        data = (handler, kwargs)
+
+        self.special_handlers[name] = data
+    
+    def set_404(self):
+        """Decorator to set the decorated function as the special handler for
+        not found pages.
+        
+        Example::
+        
+            @app.set_404()
+            def myfunction(request):
+                return Response('This page does not exist you know?')
+        """
+        def decorate(f):
+            kwargs = {'name':'404', 'function':f, 'handler':'function'}
+            self.set_special_handler(**kwargs)
+            return f
+        return decorate
+    
+    def set_404_html(self, template, **infos):
+        """Helper function to set a template as the special handler for not
+        found pages.
+        
+        Example::
+        
+            app.set_404_html('404.html')
+            
+        Or with custom context::
+        
+            app.set_404_html('404.html', VERSION='4.2')
+        """
+        kwargs = {'name': '404', 'template':template, 'handler':'template'}
+        final = kwargs
+        final.update(infos)
+        self.set_special_handler(**final)
+    
+    def set_404_markdown(self, **infos):
+        """Helper function to set a markdown file as the special handler for
+        not found pages.
+        
+        Example::
+        
+            app.set_404_markdown(template='context.html', file='data/404.md')
+        """
+        kwargs = {'name': '404', 'handler':'markup', 'markup': 'markdown'}
+        final = kwargs
+        final.update(infos)
+        self.set_special_handler(**final)
+    
     def function(self, info, request, **values):
         # Handle the request with a function
         return info['function'](request, **values)
-        
+    
     def template(self, info, request, **values):
         # Handle a request by rendering a template
         context = info.get('context', {})
         return self.render_template(info['template'], **context)
-        
+    
     def markup(self, info, request, **values):
         template = info.get('template', None)
         if template == None:
@@ -209,7 +300,7 @@ class Buro(object):
             
             
             return self.render_template(template, content=html, **context)
-        
+    
     def render_template(self, template, **context):
         """Render the given :attr:`template` file and put it in a 
         :attr:`~buro.Buro.Response` object.
@@ -219,8 +310,8 @@ class Buro(object):
         
         """
         return Response(self.jinja_env.get_template(template).render(**context))
-        
-        
+    
+    
     def url(self, url, **opts):
         """Decorator to publish a custom view (function).
         
@@ -244,7 +335,7 @@ class Buro(object):
             self.local_mapping[url] = data
             return f
         return decorate
-        
+    
     def publish_html(self, **kwargs):
         """Publish a static HTML (with Jinja2 templating system). This is a
         shortcut for ``publish(handler='template')``.
@@ -261,7 +352,7 @@ class Buro(object):
         data = kwargs
         data['handler'] = 'template'
         self.publish(**data)
-        
+    
     def publish_markdown(self, **kwargs):
         """Publish a markown file rendered in a template file (with Jinja2
         templating system). This is a shortcut for
@@ -286,7 +377,7 @@ class Buro(object):
         data['markup'] = 'markdown'
         self.publish(**data)
     
-        
+    
     def publish(self, **kwargs):
         """Put something visible on the website. You can basically add
         anything which returns a Werkzeug Response object. However some 
@@ -335,8 +426,8 @@ class Buro(object):
         
         self.local_mapping[kwargs['url']] = data
         url_map.add(Rule(kwargs['url'], endpoint=kwargs['url']))
-            
-        
+    
+    
     def run(self, host="127.0.0.1", port=8000, **options):
         """Start a local developement server. If :attr:`debug` is set, it will
         automatically reload when code changes and will print a debug page 
